@@ -8,6 +8,7 @@ ReactJS - ipywidgets relation:
 """
 import contextlib
 import copy
+import functools
 import inspect
 import logging
 import sys
@@ -42,6 +43,21 @@ from . import _version
 
 __version__ = _version.__version__
 
+
+ipywidget_version_major = int(widgets.__version__.split(".")[0])
+
+
+class _classproperty_widget_fix(object):
+    def __get__(self, owner_self, owner_cls):
+        assert owner_self is None
+        return owner_cls._active_widgets
+
+
+if ipywidget_version_major >= 8:
+    if not hasattr(widgets.Widget, "widgets"):
+        widgets.Widget.widgets = _classproperty_widget_fix()
+
+
 _last_rc = None  # used for testing
 local = threading.local()
 T = TypeVar("T")
@@ -408,6 +424,7 @@ class ComponentFunction(Component):
         self.f = f
         self.name = self.f.__name__
         self.mime_bundle = mime_bundle
+        functools.update_wrapper(self, f)
 
     def __eq__(self, rhs):
         if self is rhs:
@@ -830,10 +847,10 @@ class _RenderContext:
         assert self.context is not None
         name = debug_name or "no-name"
         if len(self.context.memo) <= self.context.memo_index:
-            self.context.memo_index += 1
             value = f()
             memo = (value, dependencies)
             self.context.memo.append(memo)
+            self.context.memo_index += 1
             logger.info("Initial memo = %r for index %r (debug-name: %r)", memo, self.context.memo_index - 1, name)
             return value
         else:
@@ -912,6 +929,8 @@ class _RenderContext:
     def render(self, element: Element, container: widgets.Widget = None):
         # render + consolidate
         widget = None
+        if container is None:
+            container = self.container
         with self.thread_lock:
             prev_rc = getattr(local, "rc", None)
             try:
@@ -1192,9 +1211,18 @@ class _RenderContext:
                     if el.is_shared:
                         # TODO: why do the tests pass if we comment the next line out
                         self._shared_elements_next.remove(el)
-                if context.effect_index != len(context.effects):
+                # if we had an exception, we allow for LESS hooks calls, since the render body might not be executed completely
+                if not ((context.effect_index == len(context.effects)) or (context.exceptions_self and context.effect_index <= len(context.effects))):
                     raise RuntimeError(
-                        f"Previously render had {len(context.effects)} effects, this run {context.effect_index} (in element/component: {el}/{el.component})"
+                        f"Previously render had {len(context.effects)} effects, this run {context.effect_index} "
+                        f"(in element/component: {el}/{el.component}). "
+                        "Are you using conditional hooks?"
+                    )
+                if not ((context.memo_index == len(context.memo)) or (context.exceptions_self and context.memo_index <= len(context.memo))):
+                    raise RuntimeError(
+                        f"Previously render had {len(context.memo)} calls to use_memo, this run {context.memo_index} "
+                        f"(in element/component: {el}/{el.component}). "
+                        "Are you using conditional hooks?"
                     )
                 # only expose to parent when no error occurs
                 context.parent.children_next[key] = context
@@ -1304,7 +1332,12 @@ class _RenderContext:
                             else:
                                 # dependencies changed, cleanup and execute next
                                 if effect.cleanup:
-                                    effect.cleanup()
+                                    try:
+                                        effect.cleanup()
+                                    except BaseException as e:
+                                        context.exceptions_self.append(e)
+                                        self._rerender_needed_reason = "Exception ocurred during effect"
+                                        self._rerender_needed = True
                                 effect = child_context.effects[effect_index] = effect.next
                                 try:
                                     effect()
@@ -1509,8 +1542,12 @@ class _RenderContext:
 
                 for effect_index, effect in enumerate(self.context.effects):
                     if effect.cleanup:
-                        effect.cleanup()
-
+                        try:
+                            effect.cleanup()
+                        except BaseException as e:
+                            context.exceptions_self.append(e)
+                            self._rerender_needed_reason = "Exception ocurred during effect"
+                            self._rerender_needed = True
                 assert self.context.root_element is not None
                 new_parent_key = join_key(parent_key, key)
                 self._remove_element(self.context.root_element, "/", parent_key=new_parent_key)
@@ -1629,7 +1666,8 @@ def display(el: Element, mime_bundle: Dict[str, Any] = mime_bundle_default):
         MIME_WIDGETS: {"version_major": 2, "version_minor": 0, "model_id": widget._model_id},
     }
     IPython.display.display(data, raw=True)
-    widget._handle_displayed()
+    if ipywidget_version_major < 8:
+        widget._handle_displayed()
 
 
 def make(el: Element, handle_error: bool = True):
