@@ -17,6 +17,7 @@ from . import logging  # noqa: F401
 from . import bqplot
 from . import ipyvuetify as v
 from . import ipywidgets as w
+from .core import ipywidget_version_major
 
 T = TypeVar("T")
 
@@ -26,7 +27,7 @@ def first(container: List[T]) -> T:
 
 
 def clear():
-    widgets.Widget.widgets = {}
+    widgets.Widget.widgets.clear()
 
 
 def count():
@@ -674,8 +675,12 @@ def test_shared_instance_via_widget_element(ButtonComponent, Container):
     assert vbox.children[0] is not vbox.children[1]
     assert vbox.children[0].description == "Button 0"
     assert vbox.children[1].description == "Button 1"
-    assert vbox.children[0].tooltip == ""
-    assert vbox.children[1].tooltip == ""
+    if ipywidget_version_major >= 8:
+        assert vbox.children[0].tooltip is None
+        assert vbox.children[1].tooltip is None
+    else:
+        assert vbox.children[0].tooltip == ""
+        assert vbox.children[1].tooltip == ""
     rc.close()
     checkbox.style.close()
     checkbox.layout.close()
@@ -1085,7 +1090,7 @@ def test_context():
 
     @react.component
     def SubChild2():
-        clicks, _dispatch = react.use_context(store_context)
+        clicks, _dispatch = react.use_context(store_context)  # type: ignore
         return w.Button(description=f"Child2: Clicked {clicks} times")
 
     @react.component
@@ -1094,7 +1099,7 @@ def test_context():
 
     @react.component
     def Child1():
-        clicks, dispatch = react.use_context(store_context)
+        clicks, dispatch = react.use_context(store_context)  # type: ignore
         return w.Button(description=f"Child1: Clicked {clicks} times", on_click=lambda: dispatch("increment"))
 
     @react.component
@@ -1600,6 +1605,8 @@ def test_exception_handler_in_render():
         fail, set_fail = react.use_state(False)
         if fail:
             raise Exception("fail")
+        # allow different amount of use_memos
+        react.use_memo(lambda: 1)
         return w.IntSlider()
 
     @react.component
@@ -1691,6 +1698,44 @@ def test_recover_exception_in_reconcilliate():
     rc.close()
 
 
+def test_recover_exception_in_cleanup():
+    set_fail = None
+
+    @react.component
+    def Test():
+        nonlocal set_fail
+        fail, set_fail = react.use_state(False)
+
+        def some_effect():
+            def cleanup():
+                if fail:
+                    raise Exception("fail")
+
+            return cleanup
+
+        react.use_effect(some_effect, [fail])
+        return w.IntSlider()
+
+    box, rc = react.render(Test())
+    assert set_fail is not None
+    assert rc._find(ipywidgets.IntSlider)
+    set_fail(True)
+    assert rc._find(ipywidgets.IntSlider)
+    # it will not fail in the cleanup, so after we set it to False
+    set_fail(False)
+    assert "Traceback" in rc._find(ipywidgets.HTML).widget.value
+    assert not rc._is_rendering
+    rc.close()
+
+    # now do the cleanup due to rendering sth else
+    box, rc = react.render(Test())
+    set_fail(True)
+    assert rc._find(ipywidgets.IntSlider)
+    rc.render(w.Button(description="Might fail because of cleanup"))
+    assert "Traceback" in rc._find(ipywidgets.HTML).widget.value
+    rc.close()
+
+
 def test_recover_exception_in_widget_update():
     set_fail = None
 
